from pycocotools.coco import COCO
import os
# 引入权限掩码模块
import stat
from ..base_labelme import BaseLabelme
import shutil
import json
from ..utils import Encoder
from tqdm import *
import numpy as np
import collections


# 继承labelme 把coco文件转
# def labelme_shapes2coco_ann(param, categories_name, cur_ann_id, cur_image_id):
#     pass

class Coco(BaseLabelme):
    def __init__(self, only_annt, images_dir=None, annotation_file=None, output_dir=None, labelme_dir=None,
                 is_labelme=False):
        # print(only_annt)
        # print(images_dir)
        # print(annotation_file)
        # print(output_dir)
        # print(labelme_dir)
        # print(is_labelme)
        if annotation_file is not None:
            self.coco = COCO(annotation_file)
        else:
            print('当前输入的coco文件为空，处理labelme转coco则正常')
        super().__init__(labelme_dir, images_dir, only_annt, is_labelme)

    def self2labelme(self):
        """
        实现把coco转labelme
        @param labelme_save_dir: 字符串，生成labelme保存路径
        @param labelme_img_dir: 字符串，生成labelme文件内容中对应的图片路径
        @param input_images_dir: 字符串，输入图片路径
        @param output_images_dir: 字符串，保存根据类别筛选后的图片路径。默认为空，不保存图片，如果传入了路径则保存
        coco转labelme遗留的问题，针对背景类的统计，没有赋值
        """
        # 获取每一张图片的id
        img_ids = self.coco.getImgIds()
        # 循环遍历每一个id
        i = 0
        for img_id in tqdm(img_ids):
            # 根据信息获取labelme注释文件路劲 273271,c9db000d5146c15
            img_info = self.coco.loadImgs(img_id)[0]
            img_path = os.path.join(self.images_dir, img_info['file_name'])
            # if self.output_images_dir is not None or '':
            #     shutil.copy(img_path, self.output_images_dir)
            img_name = os.path.basename(img_path)
            img_prefix = os.path.splitext(img_name)[-2]
            img_suffix = os.path.splitext(img_name)[-1]
            image_standard = self.images_dir + '/00.images'
            labelme_standard = self.images_dir + '/01.labelme'

            # 拼接每一个labelme文件的json路径
            # labelme_path = os.path.join(image_standard, img_prefix + '.json')
            # image_path = os.path.join(labelme_standard, img_prefix + img_suffix)
            # original_image_path = os.path.join(self.input_images_dir, img_prefix + img_suffix)
            labelme_path = os.path.join(img_prefix + '.json')
            image_path = os.path.join(self.images_dir, img_prefix + img_suffix)
            width, height = img_info['width'], img_info['height']
            # 通过注释id寻找到同一帧图片下的多个注释属性
            labelme_data = self.single_coco2labelme(img_id)
            labelme_data['imageHeight'] = height
            labelme_data['imageWidth'] = width
            labelme_data['imagePath'] = os.path.join(os.path.join('..//', '00.images'), img_name)
            data_info = dict(image_dir=image_standard,
                             image_file=image_path,
                             labelme_dir=labelme_standard,
                             labelme_file=labelme_path,
                             labelme_info=labelme_data,
                             background=False)
            if labelme_data:
                if labelme_data['shapes']:
                    for shape in labelme_data['shapes']:
                        if shape['label'] not in self.name_classes:
                            self.name_classes.append(shape['label'])
                        # self.class2datainfo[shape['label']].append(labelme_data)
                        if shape['shape_type'] not in self.shape_type:
                            self.shape_type.append(shape['shape_type'])
                        # self.type2datainfo[shape['shape_type']].append(labelme_data)
                else:  # 有labelme但shapes为空
                    self.background.append(image_path)
                    # 默认没有背景类，有背景类把background设置为True
                    data_info['background'] = True
            else:  # 存在图片木有labelme
                self.background.append(image_path)
                data_info['background'] = True
            self.data_infos.append(data_info)
            i += 1
        print("coco转labelme结束,一共%d" % i)


    def single_coco2labelme(self, img_id):
        """
        实现每一个coco注释合成labelme注释
        @param img_id:
        @return:
        """
        ann_ids = self.coco.getAnnIds(imgIds=img_id)
        labelme_data = dict(
            version='4.5.9',
            flags={},
            shapes=[],
            imagePath=None,
            imageData=None,
            imageHeight=None,
            imageWidth=None
        )
        if ann_ids:
            shapes = []
            anns = self.coco.loadAnns(ann_ids)
            for ann in anns:
                # 取到同一张图片的多个注释属性
                category_id = ann['category_id']
                # 获取坐标框
                bbox = ann['bbox']
                # 获取image_id
                # 坐标切割
                bbox = [bbox[0:2], bbox[2:4]]
                # 计算labelme的points公式
                # coco_bbox=x,y,w,h，即左上角的坐标(x,y)和宽高(w,h)
                # labelme=x1,y1,x2,y2，即：左上角的坐标(x,y)和右下角的坐标(x+w,y+h)
                # x1 = x,y1=y,x2=x+w,y2=y+h
                # points=[[x,y],[w,h]],其中,bbox[0]=[x,y],bbox[0][0] + bbox[1][0]=w,bbox[0][1] + bbox[1][1]=h
                # 左上角的坐标(x,y)右上角的坐标(x,y+h)左下角的坐标(x+w,y)右下角的坐标(x+w,y+h)
                points = [bbox[0], [bbox[0][0] + bbox[1][0], bbox[0][1] + bbox[1][1]]]
                # 通过类别id获取类别名称
                cats = self.coco.loadCats(category_id)[0]
                # 取到类别名称
                name = cats['name']
                # 临时针对某个标签修改，编写逻辑
                # if name == 'phone':
                #     labelme_name = 'oneHand'
                #     shape = {"label": labelme_name, "points": points, "group_id": None, "shape_type": "rectangle", "flags": {}}
                #     shapes.append(shape)
                shape = {"label": name, "points": points, "group_id": None, "shape_type": "rectangle", "flags": {}}
                shapes.append(shape)
            labelme_data['shapes'] = shapes
        return labelme_data

    def labelme_to_coco(self, input_dir=None, output_dir=None):
        """
        labelme转coco数据集
        :param input_dir:
        :param output_dir:
        :return:
        """
        cur_ann_id = 0
        cur_image_id = 0
        coco_images = []
        annotations = []
        categories = []
        categories_name = []
        coco_file_name = ''
        if output_dir is not None and output_dir != '':
            os.makedirs(output_dir, stat.S_IRWXU, exist_ok=True)
            coco_file_name = os.path.join(output_dir, 'coco.json')
        # global images_path
        if self.data_infos:
            for labelme_info in self.data_infos:
                cur_image_id += 1
                # 不是背景类进行labelme转coco处理
                if labelme_info['background'] is False:
                    # if self.coco_images_path is None or self.coco_images_path == '':
                    #     images_path = labelme_info['image_file']
                    # else:
                    # images_path = os.path.join(labelme_info['image_dir'], labelme_info['image_file']).replace("\\", "/")
                    images = dict(
                        file_name=labelme_info['image_file'],
                        height=labelme_info['labelme_info']['imageHeight'],
                        width=labelme_info['labelme_info']['imageWidth'],
                        id=cur_image_id,
                    )
                    coco_images.append(images)
                    one_img_ann_lsit = self.labelme_shapes2coco_ann(labelme_info['labelme_info']['shapes'],
                                                                    categories_name, cur_ann_id, cur_image_id)
                    annotations.extend(one_img_ann_lsit)
            for category_id, categorie_name in enumerate(categories_name, 1):
                categorie = dict()
                categorie['name'] = categorie_name
                categorie['id'] = category_id
                categories.append(categorie)
            coco_data = dict(
                images=coco_images,
                annotations=annotations,
                categories=categories,
            )
            # labelme转coco后无法显示中文
            with open(coco_file_name, 'w', encoding='utf-8') as coco_fp:
                # json.dump(coco_data, coco_fp, indent=4, cls=Encoder)  # indent=4 更加美观显示 indent=4 缩进 4个空格
                # 做dump与dumps操作时，会默认将中文转换为unicode，但在做逆向操作load和loads时会转换为中文，但是中间态
                json.dump(coco_data, coco_fp, ensure_ascii=False, indent=4, cls=Encoder)
        return None

    def labelme_shapes2coco_ann(self, shapes, categories_name, cur_ann_id, cur_image_id):
        """
        把labeme标签转成coco标签
        @param shapes:
        @return:
        """
        shapes_type = collections.defaultdict(list)
        for shape in shapes:
            if shape['label'] not in categories_name:
                categories_name.append(shape['label'])
            shapes_type[shape['shape_type']].append(shape)

        one_img_ann_lsit = []
        for shapes_type, shapes_type_ann in shapes_type.items():
            if shapes_type == 'rectangle':
                ann_list = self.rectangle_shapes2coco(shapes_type_ann, categories_name, cur_ann_id, cur_image_id)
            elif shapes_type == 'polygon':
                continue
                raise NotImplementedError
            else:
                continue
                raise NotImplementedError
            one_img_ann_lsit.extend(ann_list)
        return one_img_ann_lsit

    def rectangle_shapes2coco(self, shapes, categories_name, cur_ann_id, cur_image_id):
        """
        实现矩形框转换计算
        @param shapes:
        @return:
        """
        ann_list = []
        for shape in shapes:
            ann = self.get_default_ann(cur_ann_id, cur_image_id)
            category_id = categories_name.index(shape['label']) + 1
            ann['category_id'] = category_id
            points = np.array(shape['points'])
            point_min, point_max = points.min(axis=0), points.max(axis=0)
            # if points is not None and point_min is not None and point_max is not None:
            w, h = point_max - point_min
            ann['bbox'] = [point_min[0], point_min[1], w, h]
            ann['area'] = w * h
            ann_list.append(ann)
        return ann_list

    def get_default_ann(self, cur_ann_id, cur_image_id):
        """
        标签属性定义，并id自增
        @return:
        """
        cur_ann_id += 1
        annotation = dict(
            id=cur_ann_id,
            image_id=cur_image_id,
            segmentation=[],
            area=0,
            bbox=[],
            iscrowd=0,
        )
        return annotation



from .coder import Encoder
# from .path import get_valid_paths, get_multiple_path
# __all__ = ['Encoder', 'get_valid_paths', 'get_multiple_path']

from .path import get_valid_paths
__all__ = ['Encoder', 'get_valid_paths']




import argparse
import ast
import ccdt.dataset as ccdt
import ccdt.dataset.utils.coder
import ccdt.dataset.utils.path


def parser_args():
    parser = argparse.ArgumentParser()
    # 分组是给用户看的，看什么呐？
    # rename_parser = parser.add_argument_group()
    # 如果属于filter功能，就把传入的筛选类别进行分组
    # filter_parser = parser.add_argument_group()
    parser.add_argument('--input_datasets', type=ast.literal_eval, help="输入labelme数据集列表字典路径")
    parser.add_argument('--output_dir', type=str, help="输入抠图保存路径")
    parser.add_argument('--input_dir',  type=str, help="输入替换路径")
    parser.add_argument('--output_format', type=str, help="输入输出格式")
    parser.add_argument('--function', type=str,
                        help="输入操作功能参数:print,convert,filter,matting,rename,visualize,merge只能输入单个")
    parser.add_argument('--filter_category', type=ast.literal_eval, help="输入类别筛选参数，单个与多个都可以输入")
    # 当不输入--only_annotation，默认为False；输入--only_annotation，才会触发True值。False处理labelme和图片，True只处理labelme
    parser.add_argument('--only_annotation', action="store_true", help="默认False，处理图片和注释文件。传参则设置为True，只处理注释文件")
    parser.add_argument('--filter_type_shapes', type=ast.literal_eval, help="输入形状筛选参数，单个与多个都可以输入")
    parser.add_argument('--input_coco_file', type=str, help="输入形状筛选参数，单个与多个都可以输入")
    parser.add_argument('--rename_category', type=ast.literal_eval, help="输入修改标签类别参数")
    parser.add_argument('--filter_empty', action="store_true", help="默认False，保留背景类。传参则设置为True，不保留背景类")
    parser.add_argument('--only_empty', action="store_true", help="默认False，不保留背景类。传参则设置为True，只保留背景类")
    parser.add_argument('--only_annt', action="store_true", help="默认False，处理coco注释文件和图片。传参则设置为True，只处理注释文件")
    parser.add_argument('--filter_flags', type=ast.literal_eval, help="类别属性筛选，输入类别属性字典列表。比如person类下有，手、脚、头")
    args = parser.parse_args()
    # 如果需要进行类别过滤，则必须要有操作功能filter参数存在
    if args.filter_category is not None and args.function == 'filter':
        return args
    # 重命名
    elif args.rename_category is not None and args.function == 'rename':
        return args
    # labelme转coco，coco转labelme
    elif args.function == 'convert':
        return args
    # 抠图，单数据集、多数据集
    elif args.function == 'matting':
        return args
    # 可视化
    elif args.function == 'visualize':
        return args
    # 合并类别筛选数据
    elif args.function == 'merge':
        return args
    else:
        assert not args.function, '传入的操作功能参数不对:{}'.format(args.function)

    # return args


def load_datasets(datasets_info):
    args = parser_args()
    datasets = []
    for dataset_info in datasets_info:
        # 使用labelme类加载
        if dataset_info['format'] == 'labelme' and args.output_format == 'labelme':
            try:
                dataset = ccdt.BaseLabelme(dataset_info['labelme_dir'], dataset_info['images_dir'],args.only_annotation)
                datasets.append(dataset)
            except Exception as e:
                print(e)
            continue
            # 把整个对象对象追加到列表中
            # dataset = ccdt.BaseLabelme(dataset_info['labelme_dir'], dataset_info['images_dir'], args.only_annotation)
            # datasets.append(dataset)
            print('labelme数据集加载成功')
        # 使用coco类加载
        if dataset_info['format'] == 'labelme' and args.output_format == 'coco':
            # 指定传参写法，labelme_dir=dataset_info['labelme_dir'],如果只实现labelme数据集，is_labelme=True，默认为False
            dataset = ccdt.Coco(False, images_dir=dataset_info['images_dir'], labelme_dir=dataset_info['labelme_dir'],
                                is_labelme=True)
            datasets.append(dataset)
            print("labelme转coco数据加载成功")
        # 使用coco类加载
        if dataset_info['format'] == 'coco' and args.output_format == 'coco':
            dataset = ccdt.Coco(False, dataset_info['images_dir'], dataset_info['coco_file'])
            datasets.append(dataset)
        # 使用coco类加载
        if dataset_info['format'] == 'coco' and args.output_format == 'labelme':
            print(args.only_annt)
            dataset = ccdt.Coco(args.only_annt, images_dir=dataset_info['images_dir'],
                                annotation_file=dataset_info['coco_file'])
            datasets.append(dataset)
            print("coco转labelme数据加载成功")
    return datasets


def main():
    args = parser_args()
    global datasets
    # print(args)
    # 1.加载数据集
    if args.input_datasets is not None:
        # 如果输入的字典，就转成列表嵌套字典，如果输入的列表嵌套字典，就直接进行数据封装
        if isinstance(args.input_datasets, tuple):
            # args.input_datasets = [args.input_datasets]  # 字典转列表
            args.input_datasets = list(args.input_datasets)  # 元组转列表
        for dataset_info in args.input_datasets:
            # 如果输入路径为空，报错
            if dataset_info.get('input_dir', False) == '':
                assert dataset_info['input_datasets'] == '', '{}路径为空'.format(dataset_info['input_datasets'])
            if dataset_info['format'] != 'labelme' or dataset_info['format'] == '':
                assert dataset_info['format'] == 'labelme', '{}不是labelme或者为空'.format(dataset_info['format'])
            # 如果输入路径不为空，且格式正确,直接加载数据集
            if not dataset_info.get('input_dir', False):
                datasets = load_datasets(args.input_datasets)
            # 如果输入路径不为空，且格式不正确需要封装路径返回
            if dataset_info.get('input_dir', False):
                labelme_path_list = ccdt.utils.path.get_labelme_path(dataset_info['input_dir'], dataset_info['format'])
                datasets = load_datasets(labelme_path_list)
    else:
        print('当前数据集为空')
    # print(datasets)
    # 打印输出，首次数据集加载
    for dataset in datasets:
        print('Labelme 数据集图像路径:%s' % dataset.self_bak.images_dir, 'Labelme 数据集注释路径:%s' % dataset.self_bak.labelme_dir)
        print(dataset)
    # 多线程变量.如果是IO密集型任务，使用多线程，如果是CPU密集型任务，使用多进程;
    # labelme_pool = multiprocessing.Pool(8)
    # print(multiprocessing.cpu_count())
    # print(multiprocessing.current_process())
    # queue = multiprocessing.Manager().Queue()
    # 2.对数据进行处理
    if args.function == 'matting':
        for dataset in datasets:
            # print(dataset)
            dataset.crop_objs(args.output_dir, replaces={args.input_dir: ''}, min_pixel=512, )
            # print('抠图完成')
            # print('当前抠图总数:%r' % dataset.num_crop_images)
    elif args.function == 'merge':
        merge_dataset = datasets[0]
        # 循环累加标签及类别，datasets[1:]表示从列表中第一个开始取
        for dataset in datasets[1:]:
            # 第0个和第一个相加后，赋值给一个变量，然后把这个变量看作一个整体，继续加第三个。
            merge_dataset = merge_dataset + dataset
        merge_dataset.save_labelme(args.output_dir, replaces={args.input_dir: ''})
        # print('类别合并完成')
    elif args.function == 'convert':
        # coco转labelme 和labelme转coco是否都写在这里
        if args.output_format == 'labelme':
            for dataset in datasets:
                # print(dataset)
                dataset.save_labelme(args.output_dir, replaces={args.input_dir: ''})
            # print('coco转labelme完成')
        elif args.output_format == 'coco':
            # print('labelme转coco开始')
            for dataset in datasets:
                dataset.labelme_to_coco(args.input_dir, args.output_dir)
            # print('labelme转coco完成')
    elif args.function == 'rename':
        # print(args.rename_category)
        for dataset in datasets:
            # print(dataset)
            dataset.rename(args.rename_category)
            dataset.save_labelme(args.output_dir, replaces={args.input_dir: ''})
    elif args.function == 'visualize':
        for dataset in datasets:
            dataset.visualization(args.output_dir, replaces={args.input_dir: ''})
    elif args.function == 'filter':
        for dataset in datasets:
            # print(dataset)
            # print(args.filter_empty)
            # print(args.filter_flags)
            # args.filter_empty：True默认背景类不需要,False是需要。args.only_empty：只筛选背景类。针对抽取过程中已经变成背景类的情况怎么办？
            dataset = dataset(args.filter_empty, args.only_empty, name_classes=args.filter_category,
                              type_shapes=args.filter_type_shapes, filter_flags=args.filter_flags)
            # 保障有返回，同时之前的没有改变
            # print(dataset_handle)
            # print(dataset)
            dataset.save_labelme(args.output_dir, replaces={args.input_dir: ''})
            # 调用之后，是增加，之前的统计还在，如何清空
            # print(dataset.__update_property__())
    # 3.打印输出每次功能实现后的属性
    for dataset in datasets:
        print('Labelme 数据集图像路径:%s' % dataset.self_bak.images_dir, 'Labelme 数据集注释路径:%s' % dataset.self_bak.labelme_dir)
        print(dataset.self_bak)
    print('数据处理完毕')


if __name__ == '__main__':
    main()



import json
import numpy as np


class Encoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, np.integer):
            return int(obj)
        elif isinstance(obj, np.floating):
            return float(obj)
        elif isinstance(obj, np.ndarray):
            return obj.tolist()
        else:
            return super(Encoder, self).default(obj)



from pycocotools.coco import COCO
import os
# 引入权限掩码模块
import stat
from ..base_labelme import BaseLabelme
import shutil
import json
from ..utils import Encoder
from tqdm import *
import numpy as np
import collections


# 继承labelme 把coco文件转
class Coco(BaseLabelme):
    def __init__(self, filter_condition, data_infors, coco_dir='./coco', replaces=None, coco_file=None,
                 transform_save_dir=None, image_dir=None, labelme_dir=None, input_images_dir=None,
                 output_images_dir=None):
        print(replaces)
        self.data_infors = data_infors
        self.coco = COCO(coco_file)
        if input_images_dir is not None and input_images_dir != '':
            os.makedirs(input_images_dir, stat.S_IRWXU, exist_ok=True)
        if coco_dir is not None and coco_dir != '':
            os.makedirs(coco_dir, stat.S_IRWXU, exist_ok=True)
            self.user_customize_coco_path = os.path.join(coco_dir, 'coco.json')
        if transform_save_dir is not None and transform_save_dir != '':
            images_dir = os.path.join(transform_save_dir, labelme_dir)
            os.makedirs(images_dir, stat.S_IRWXU, exist_ok=True)
            self.labelme_save_dir = images_dir
        if output_images_dir is not None and output_images_dir != '':
            os.makedirs(output_images_dir, stat.S_IRWXU, exist_ok=True)
        if image_dir is not None and image_dir != '':
            labelme_dir = os.path.join(transform_save_dir, image_dir)
            os.makedirs(labelme_dir, stat.S_IRWXU, exist_ok=True)
            self.labelme_img_dir = labelme_dir
        self.input_images_dir = input_images_dir
        self.output_images_dir = output_images_dir
        self.annotations = []
        self.categories = []
        self.labelmes_info = []
        self.cur_image_id = 0
        self.cur_ann_id = 0
        self.categories_name = []
        self.images = []
        # 相对路径和绝对路径参数设置
        self.coco_images_path = replaces
        self.filter_condition = filter_condition
    # 这个方法被继承后，只是为了获取labelme处理过后的datainfos变量信息。具体实现在子类，父类不实现。
    def self2labelme(self):
        """
        实现把coco转labelme
        @param labelme_save_dir: 字符串，生成labelme保存路径
        @param labelme_img_dir: 字符串，生成labelme文件内容中对应的图片路径
        @param input_images_dir: 字符串，输入图片路径
        @param output_images_dir: 字符串，保存根据类别筛选后的图片路径。默认为空，不保存图片，如果传入了路径则保存
        """
        # 获取每一张图片的id
        img_ids = self.coco.getImgIds()
        # 循环遍历每一个id
        i = 0
        for img_id in tqdm(img_ids):
            # 根据信息获取labelme注释文件路劲 273271,c9db000d5146c15
            img_info = self.coco.loadImgs(img_id)[0]
            img_path = os.path.join(self.input_images_dir, img_info['file_name'])
            if self.output_images_dir is not None or '':
                shutil.copy(img_path, self.output_images_dir)
            img_name = os.path.basename(img_path)
            img_prefix = os.path.splitext(img_name)[-2]
            img_suffix = os.path.splitext(img_name)[-1]
            # 拼接每一个labelme文件的json路径
            labelme_path = os.path.join(self.labelme_save_dir, img_prefix + '.json')
            image_path = os.path.join(self.input_images_dir, img_prefix + img_suffix)
            if self.filter_condition is False:
                shutil.copy(image_path, self.labelme_img_dir)
            width, height = img_info['width'], img_info['height']
            # 通过注释id寻找到同一帧图片下的多个注释属性
            labelme_data = self.single_coco2labelme(img_id)
            labelme_data['imageHeight'] = height
            labelme_data['imageWidth'] = width
            if self.coco_images_path is None or self.coco_images_path == '':
                labelme_data['imagePath'] = os.path.join(self.labelme_img_dir, img_name)
            else:
                labelme_data['imagePath'] = os.path.join(self.labelme_img_dir.replace(self.coco_images_path, '..'), img_name).replace("\\", "/")
            with open(labelme_path, 'w', encoding='UTF-8') as labelme_fp:
                json.dump(labelme_data, labelme_fp, indent=4, cls=Encoder)
            i += 1
        print("coco转labelme结束,一共%d" % i)

    def single_coco2labelme(self, img_id):
        """
        实现每一个coco注释合成labelme注释
        @param img_id:
        @return:
        """
        ann_ids = self.coco.getAnnIds(imgIds=img_id)
        labelme_data = dict(
            version='4.5.9',
            flags={},
            shapes=[],
            imagePath=None,
            imageData=None,
            imageHeight=None,
            imageWidth=None
        )
        if ann_ids:
            shapes = []
            anns = self.coco.loadAnns(ann_ids)
            for ann in anns:
                # 取到同一张图片的多个注释属性
                category_id = ann['category_id']
                # 获取坐标框
                bbox = ann['bbox']
                # 获取image_id
                # 坐标切割
                bbox = [bbox[0:2], bbox[2:4]]
                # 计算labelme的points公式
                # coco_bbox=x,y,w,h，即左上角的坐标(x,y)和宽高(w,h)
                # labelme=x1,y1,x2,y2，即：左上角的坐标(x,y)和右下角的坐标(x+w,y+h)
                # x1 = x,y1=y,x2=x+w,y2=y+h
                # points=[[x,y],[w,h]],其中,bbox[0]=[x,y],bbox[0][0] + bbox[1][0]=w,bbox[0][1] + bbox[1][1]=h
                # 左上角的坐标(x,y)右上角的坐标(x,y+h)左下角的坐标(x+w,y)右下角的坐标(x+w,y+h)
                points = [bbox[0], [bbox[0][0] + bbox[1][0], bbox[0][1] + bbox[1][1]]]
                # 通过类别id获取类别名称
                cats = self.coco.loadCats(category_id)[0]
                # 取到类别名称
                name = cats['name']
                # 临时针对某个标签修改，编写逻辑
                # if name == 'phone':
                #     labelme_name = 'oneHand'
                #     shape = {"label": labelme_name, "points": points, "group_id": None, "shape_type": "rectangle", "flags": {}}
                #     shapes.append(shape)
                shape = {"label": name, "points": points, "group_id": None, "shape_type": "rectangle", "flags": {}}
                shapes.append(shape)
            labelme_data['shapes'] = shapes
        return labelme_data

    def labelme_to_coco(self):
        self.handle_labelme_2coco_attribute()

    def handle_labelme_2coco_attribute(self):
        """
        labelme标签属性转coco标签属性
        :param labelmes_info_attribute:
        :return:
        """
        global images_path
        if self.data_infors.data_infos:
            for labelme_info in self.data_infors.data_infos:
                self.cur_image_id += 1
                # 不是背景类进行labelme转coco处理
                if labelme_info['background'] is False:
                    if self.coco_images_path is None or self.coco_images_path == '':
                        images_path = labelme_info['image_file']
                    else:
                        images_path = os.path.join(labelme_info['image_dir'], labelme_info['image_file']).replace("\\",
                                                                                                                  "/")
                    images = dict(
                        file_name=images_path,
                        height=labelme_info['labelme_info']['imageHeight'],
                        width=labelme_info['labelme_info']['imageWidth'],
                        id=self.cur_image_id,
                    )
                    self.images.append(images)
                    one_img_ann_lsit = self.labelme_shapes2coco_ann(labelme_info['labelme_info']['shapes'])
                    self.annotations.extend(one_img_ann_lsit)
            for category_id, categorie_name in enumerate(self.categories_name, 1):
                categorie = dict()
                categorie['name'] = categorie_name
                categorie['id'] = category_id
                self.categories.append(categorie)

            coco_data = dict(
                images=self.images,
                annotations=self.annotations,
                categories=self.categories,
            )
            # labelme转coco后无法显示中文
            with open(self.user_customize_coco_path, 'w', encoding='utf-8') as coco_fp:
                # json.dump(coco_data, coco_fp, indent=4, cls=Encoder)  # indent=4 更加美观显示 indent=4 缩进 4个空格
                # 做dump与dumps操作时，会默认将中文转换为unicode，但在做逆向操作load和loads时会转换为中文，但是中间态
                json.dump(coco_data, coco_fp, ensure_ascii=False, indent=4, cls=Encoder)
        return None

    def labelme_shapes2coco_ann(self, shapes):
        """
        把labeme标签转成coco标签
        @param shapes:
        @return:
        """
        shapes_type = collections.defaultdict(list)
        for shape in shapes:
            if shape['label'] not in self.categories_name:
                self.categories_name.append(shape['label'])
            shapes_type[shape['shape_type']].append(shape)

        one_img_ann_lsit = []
        for shapes_type, shapes_type_ann in shapes_type.items():
            if shapes_type == 'rectangle':
                ann_list = self.rectangle_shapes2coco(shapes_type_ann)
            elif shapes_type == 'polygon':
                continue
                raise NotImplementedError
            else:
                continue
                raise NotImplementedError
            one_img_ann_lsit.extend(ann_list)
        return one_img_ann_lsit

    def rectangle_shapes2coco(self, shapes):
        """
        实现矩形框转换计算
        @param shapes:
        @return:
        """
        ann_list = []
        for shape in shapes:
            ann = self.get_default_ann()
            category_id = self.categories_name.index(shape['label']) + 1
            ann['category_id'] = category_id
            points = np.array(shape['points'])
            point_min, point_max = points.min(axis=0), points.max(axis=0)
            # if points is not None and point_min is not None and point_max is not None:
            w, h = point_max - point_min
            ann['bbox'] = [point_min[0], point_min[1], w, h]
            ann['area'] = w * h
            ann_list.append(ann)
        return ann_list

    def get_default_ann(self):
        """
        标签属性定义，并id自增
        @return:
        """
        self.cur_ann_id += 1
        annotation = dict(
            id=self.cur_ann_id,
            image_id=self.cur_image_id,
            segmentation=[],
            area=0,
            bbox=[],
            iscrowd=0,
        )
        return annotation



import os
from ..utils import path
import json
import collections
import cv2
# import cv2
import numpy as np
from tqdm import *
import shutil
import warnings
import prettytable as pt
from ..utils import Encoder
import copy
from functools import wraps
import multiprocessing
from functools import reduce


# 定义类时，在没有确切的继承之前，默认继承object。python3默认继承object，如此定义可以在python2和python3下运行。


class BaseLabelme(object):

    @property
    def num_labelme(self):
        return len(self.labelme_paths)

    @property
    def num_images(self):
        return len(self.images_paths)

    @property
    def num_classes(self):
        return len(self.name_classes)

    @property
    def num_type(self):
        try:
            self.shape_type
        except Exception as e:
            print(e)
            return None
        return len(self.shape_type)

    @property
    def num_background(self):
        return len(self.background)

    @property
    def num_flags(self):
        return len(self.flags)

    def __update_property__(self):
        """
        统计，图片文件个数，labelme文件个数，标注形状个数，背景图片个数，标注类别个数
        """
        self.name_classes = list()
        # self.shape_type = list()
        self.background = list()
        self.labelme_paths = list()
        self.images_paths = list()
        self.flags = dict()

        for data_info in self.data_infos:
            # print(data_info)
            try:
                image_path = os.path.join(data_info['image_dir'], data_info['image_file'])
            except Exception as e:
                print(e)
                print(data_info)
                print('json文件存在，但图片文件不存在请检查数据')
                # a = os.path.join(data_info['labelme_dir'], data_info['labelme_file'])
                # print(a)
                # os.remove(a)
                continue
            # print(image_path)
            # print(data_info['image_dir'])
            # print(data_info['image_file'])
            if data_info['labelme_file']:
                json_path = os.path.join(data_info['labelme_dir'], data_info['labelme_file'])
                self.labelme_paths.append(json_path)
            else:
                json_path = None
            self.images_paths.append(image_path)
            if json_path:
                # data_info['labelme_info'] 直接内存中获取，不用读取文件
                # with open(json_path, 'r', encoding='UTF-8') as labelme_fp:
                # labelme_info = json.load(labelme_fp)
                # 直接从内存中进行获取操作
                self.shape_type = list()

                if data_info['labelme_info']['shapes']:
                    for shape in data_info['labelme_info']['shapes']:
                        # print(shape)

                        if shape['label'] not in self.name_classes:
                            self.name_classes.append(shape['label'])
                            # isinstance(self.flags, dict)
                            # print(shape['flags'] ,dict)
                        # for i in shape['flags']:
                        # print(i)
                        # if shape['flags'] not in self.flags:
                        self.flags.update(shape['flags'])
                        if shape['shape_type'] not in self.shape_type:
                            self.shape_type.append(shape['shape_type'])
                else:
                    self.background.append(image_path)
                    # 默认没有背景类，有背景类把background设置为True
                    data_info['background'] = True
            else:
                self.background.append(image_path)
                data_info['background'] = True
        # print(self.flags)
        # print()

    @classmethod
    def find(cls, image_file, data_infos):
        # 判断image_file 是否在data_infos里面，如果在就返回下标索引，如果不在返回None用于追加判断
        for index, info in enumerate(data_infos):
            # 如果多种数据集中存在相同的图片，则不能类别合并
            if info['image_file'] == image_file:
                return index
        return None

    def __repr__(self):
        """
        对象的字符串表示形式的内置方法，主要使用表格打印信息
        :return:
        """
        tb = pt.PrettyTable()
        tb.field_names = ['num_images', 'num_type', 'num_classes', 'num_labelme', 'num_background', 'flags']

        tb.add_row(
            [self.num_images, self.num_type, self.num_classes, self.num_labelme, self.num_background, self.num_flags])
        # tb.set_style(pt.MSWORD_FRIENDLY)
        return str(tb)

    def __init__(self, labelme_dir, images_dir, only_annt=False, is_labelme=False):
        # print(only_annt)
        # print(labelme_dir)
        # print(images_dir)
        """
        类实例方法。称为构造方法。初始化方法。
        :param datasets: 传入处理数据集
        :param only_labelme: False默认处理labelme和图片，True只处理labelme
        """
        self.only_annt = only_annt
        self.labelme_dir = labelme_dir
        self.images_dir = images_dir

        # self.name_classes = list()
        # self.shape_type = list()
        # self.flags = dict()
        # self.background = list()
        # self.labelme_paths = list()
        # self.images_paths = list()

        # 标签统计
        # self.label = []

        self.data_infos = list()
        self.class2datainfo = collections.defaultdict(list)
        self.type2datainfo = collections.defaultdict(list)
        # 不同元素
        self.different = list()
        # 类别和形状筛选是否删除背景类条件，False不删除，True删除
        self.filter_empty = None
        # 如果处理labelme走if，如果是子类继承走else
        self.self_bak = None
        # 通过类别判断，是子类还是父类
        if type(self) == BaseLabelme or is_labelme:
            # self.datasets = self._check_dataset(datasets)
            self.data_paths = self.get_data_paths()
            self.data_infos = self.load_labelme()
        # 其它数据集处理逻辑，比如coco转labelme
        # labelme转coco不能走这个方法，怎么办
        else:
            self.self2labelme()
        # 在coco转labelme完成后，才调用它实现它，打印相关属性信息。保持成labelme后在读取文件进行属性统计操作，还是在内存中就行属性统计操作呐
        self.__update_property__()
        # 数据初始完成后进行深度拷贝一份
        self.self_bak = copy.deepcopy(self)

    def get_data_paths(self):
        """
        数据路径处理
        :return:
        """
        data_paths = list()
        # 传递图片路径并返回图片路径列表，通过字典键（dataset['images_dir']）取到字典列表
        images_name_list = path.get_valid_paths(self.images_dir, ['.png', '.jpg', '.jpeg', '.tiff', '.psd', '.JPEG'])
        if len(images_name_list) == 0:
            print(self.images_dir + ': images图像数目={}'.format(len(images_name_list)))
        images_name_dict = dict()
        if not self.only_annt:
            for image_name in images_name_list:
                image_path = os.path.join(self.images_dir, image_name)
                # print(image_path)
                img_prefix, img_suffix = os.path.splitext(image_name)[-2], os.path.splitext(image_name)[-1]
                images_name_dict[img_prefix] = img_suffix
                #
                # self.images_paths.append(image_path)

        labelme_name_list = path.get_valid_paths(self.labelme_dir, ['.json'])
        labelme_name_dict = dict()
        for json_name in labelme_name_list:
            json_path = os.path.join(self.labelme_dir, json_name)
            # print(json_path)
            json_prefix, json_suffix = os.path.splitext(json_name)[-2], os.path.splitext(json_name)[-1]
            labelme_name_dict[json_prefix] = json_suffix
            #
            # self.labelme_paths.append(json_path)
        # print('Labelme 数据集图像:%s' % self.images_dir, '共%d张图像' % len(images_name_list),
        #       'Labelme 数据集注释:%s' % self.labelme_dir, '共%d个labelme注释文件' % len(labelme_name_list))

        # 针对没有图片或者没有labelme的文件夹，是否需要不执行？有图片，没有对应的labelme是允许的
        # assert len(images_name_list) and len(labelme_name_list), \
        # '{} 没有图片，\n{} 没有labelme'.format(self.labelme_dir, self.images_dir)

        # 把字典转list后合并成一个list,并去重。同时，集合转列表，然后再排序
        names = list(set(list(images_name_dict.keys()) + list(labelme_name_dict.keys())))
        names.sort()
        for name in names:
            data_path = dict(image_dir=self.images_dir,
                             image_file=name + images_name_dict[name] if images_name_dict.get(name,
                                                                                              False) else None,
                             labelme_dir=self.labelme_dir,
                             labelme_file=name + labelme_name_dict[name] if labelme_name_dict.get(name,
                                                                                                  False) else None)
            assert (data_path['labelme_file'] is not None) or (data_path['image_file'] is not None), \
                '{} 没有图片，\n{} 没有labelme'.format(data_path['labelme_dir'], data_path['image_dir'])
            data_paths.append(data_path)
        return data_paths

    def load_labelme(self):
        """
        读取labelme并统计背景类，有图片没有labelme和有labelme没有shapes属性则算背景类
        :return:
        """
        data_infos = list()
        # print('=' * 150)
        print("处理labelme数据进度:")
        for data_path in tqdm(self.data_paths):
            data_info = dict(image_dir=None, image_file=None, labelme_dir=None, labelme_file=None, labelme_info=None,
                             background=False)
            # 如果labelme为空，就返回空并且不拼接
            labelme_path = os.path.join(data_path['labelme_dir'], data_path['labelme_file']) if data_path[
                'labelme_file'] else None
            image_path = os.path.join(data_path['image_dir'], data_path['image_file']) if data_path[
                'image_file'] else None
            # 图像个数和labelme个数是否有为0的
            data_info['image_dir'] = data_path['image_dir']
            data_info['labelme_dir'] = data_path['labelme_dir']
            data_info['image_file'] = data_path['image_file']

            # data[image_path.replace()]

            # 运行过程中突然网络断开，执行立即出错
            if labelme_path:
                with open(labelme_path, 'r', encoding='UTF-8') as labelme_fp:
                    labelme_info = json.load(labelme_fp)
                data_info['labelme_file'] = data_path['labelme_file']
                data_info['labelme_info'] = labelme_info
                if labelme_info['shapes']:
                    for shape in labelme_info['shapes']:
                        # if shape['label'] not in self.name_classes:
                        #     #
                        #     self.name_classes.append(shape['label'])
                        self.class2datainfo[shape['label']].append(data_info)
                        # if shape['shape_type'] not in self.shape_type:
                        #     #
                        #     self.shape_type.append(shape['shape_type'])
                        self.type2datainfo[shape['shape_type']].append(data_info)
                # else:  # 有labelme但shapes为空
                #     #
                #     self.background.append(data_path)
                #     # 默认没有背景类，有背景类把background设置为True
                #     data_info['background'] = True
            # else:  # 存在图片木有labelme
            #     #
            #     self.background.append(image_path)
            #     data_info['background'] = True
            data_infos.append(data_info)
        return data_infos

    def crop_rectangle(self, image, shape):
        """
        长方形截取计算过程
        :param image: 图像
        :param shape: 形状坐标
        :return:
        """
        h = image.shape[0]
        w = image.shape[1]
        # 把从任意角度标注兼容
        points = np.array(shape['points'])
        point_min, point_max = points.min(axis=0), points.max(axis=0)
        x1, y1 = int(max(0, min(point_min[0], w))), int(max(0, min(point_min[1], h)))
        x2, y2 = int(max(0, min(point_max[0], w))), int(max(0, min(point_max[1], h)))
        # y1:y2 x1:x2,针对负数框在图片的外面时截取不到,正常标注不会超出图片面积范围。max(0, min(x, img_info['width'])把负数变成0。np.clip(point_min)
        crop_obj = image[y1:y2, x1:x2]
        return crop_obj

    def crop_objs(self, out_dir, min_pixel=10, replaces={}, ):
        # def crop_objs(self, out_dir, min_pixel=10, replaces={}, labelme_pool, queue):
        """
        截取图像功能实现，一张图片画框多少，就扣多少，不管是否重叠
        :param labelme_pool:
        :param out_dir: 保存截取图像路径
        :param shapes_type:标注形状类型，比如rectangle(长方形)、circle(圆)、polygon(多边形)、line(线)
        :param min_pixel:保存截取图片最小像素设置
        :param replaces:替换路径，把不变的路径替换为空
        """
        # 抠图数量统计
        num_crop = list()
        global folder
        assert not self.only_annt, '传入的图片路径为空，不能进行图片截取：{}'.format(self.only_annt)
        assert isinstance(replaces, dict)
        print("截图图片进度:")
        gl_tmp_out_dir: str = ''
        for data_info in tqdm(self.data_infos):
            labelme_path = os.path.join(data_info['labelme_dir'], data_info['labelme_file']) if data_info[
                'labelme_file'] else None
            image_path = os.path.join(data_info['image_dir'], data_info['image_file']) if data_info[
                'image_file'] else None
            # 只有labelme没有图片的时候不截取,直接跳过
            if (image_path is None) or (labelme_path is None):
                continue
            image = cv2.imdecode(np.fromfile(image_path, dtype=np.uint8), 1)
            img_prefix, img_suffix = os.path.splitext(data_info['image_file'])[-2], \
                                     os.path.splitext(data_info['image_file'])[-1]
            num_obj = 0
            for old, new in replaces.items():
                replace_image_path = data_info['image_dir'].replace(old, new)
                # 3. 去掉开头的 斜杠
                remove_slash = replace_image_path.strip('\\/')
                gl_tmp_out_dir = os.path.join(out_dir, remove_slash)
                folder = remove_slash.split('/')
            cut_img_name = '{}_{}'.format(folder[0], img_prefix)
            for shape in data_info['labelme_info']['shapes']:
                num_obj += 1
                # 原始图片就有-与_,把-替换成下划线，
                # obj_file = img_prefix.replace('-', '').replace('_', '') + '_{:0>6d}'.format(num_obj) + img_suffix
                # 针对新需求-不替换下划线
                obj_file = cut_img_name.replace('.', '') + '_{:0>6d}'.format(num_obj) + img_suffix
                final_out_dir = os.path.join(gl_tmp_out_dir, shape['label'])
                # exist_ok=True，这样如果文件夹存在，会忽略创建文件夹
                os.makedirs(final_out_dir, exist_ok=True)
                # print(final_out_dir)
                crop_path = os.path.join(final_out_dir, obj_file)
                num_crop.append(crop_path)
                crop_obj = self.crop_rectangle(image, shape)
                if crop_obj.size == 0:
                    print("当前文件标注存在异常，路径如下所示:")
                    print(crop_path)
                # 默认像素小于10，就不进行截取，可以自动设置
                cv2.imencode(img_suffix, crop_obj)[1].tofile(crop_path)
                # if crop_obj.shape[0] * crop_obj.shape[1] > min_pixel:
                #     queue.put([crop_obj, crop_path, img_suffix])
                # # 多线程处理，抠图保存
                # while True:
                #     labelme_pool.apply_async(self.save, args=(queue,))
                #     if queue.qsize() > 50:
                #         continue
                #     else:
                #         break

    def _del_class(self, data_infos, name_classes, filter_flags):
        """
        根据类别进行筛选，根据类别属性进行筛选，根据形状进行筛选
        :param data_infos:
        :param name_classes:
        :param filter_flags:
        :return:
        """

        def shape(data_info):
            # print(data_info)
            # 满足条件的置空
            if data_info.get('labelme_info') is not None:
                name_classes_list = list()
                for shape_element in data_info.get('labelme_info').get('shapes'):
                    # 如果筛选的类别不是要筛选的类别，就追加到列表中然后删除。
                    if shape_element['label'] not in name_classes:
                        name_classes_list.append(shape_element)
                    # 筛选类别的属性，是只筛选打勾的，并且属于筛选范围的
                    for flag in list(shape_element['flags']):  # 把字典进行循环，取键的值
                        if shape_element['flags'][flag]:  # 通过字典的键获取字典的值
                            if flag in filter_flags:  # 判断键的值是否在输入的列表中
                                pass
                            else:
                                del shape_element['flags'][flag]
                        else:
                            del shape_element['flags'][flag]
                for i in name_classes_list:
                    data_info.get('labelme_info').get('shapes').remove(i)
                # 针对按类别筛选后变成背景类的过滤。如果要保存背景类，筛选后变成背景类的数据没有保存。
                if data_info['labelme_info'].get('shapes').__len__() == 0 and data_info['background'] is False:
                    data_info['background'] = True
                if data_info['background'] and self.filter_empty is True:
                    return
                if data_info['background'] and self.filter_empty is False:
                    return data_info
                else:
                    return data_info

            if self.filter_empty is False and data_info.get('labelme_info') is None:
                return data_info

        # 没有这一步，过滤后等于没有过滤
        self.data_infos = list(filter(shape, data_infos))

    def _del_type(self, data_info, shapes_type):
        """
        shapes_type=['rectangle']
        根据标注形状进行筛选，比如rectangle(长方形)、circle(圆)、polygon(多边形)、line(线)
        :param type_shapes:
        """
        shapes_type = [shapes_type] if isinstance(shapes_type, str) else shapes_type
        for shape in data_info['labelme_info']['shapes']:
            if shape is None:
                if shape['shape_type'] not in shapes_type:
                    shape.clear()

    # 保存，传入保存路径，类别。客户想保存才保存
    def save_labelme(self, out_dir='./out_dir', replaces={}):
        """
        根据类别保存labelme。coco转labelme需要现在内存中与datainfos关联上，然后用该方法存储到磁盘上。
        :param out_dir:
        """
        if out_dir == '' or out_dir is None:
            print('传入的save_path={} 为空'.format(out_dir))
            raise ValueError
        if out_dir is not None and out_dir != '':
            os.makedirs(out_dir, exist_ok=True)
        print('类别保存处理进度:')
        global modify_labelme_dir, archive_json_path, archive_image_path
        for data_info in tqdm(self.data_infos):
            # 批量处理多文件夹数据集，把旧路径替换成自定义路径
            for old, new in replaces.items():
                replace_image_path = data_info['image_dir'].replace(old, new)
                replace_json_path = data_info['labelme_dir'].replace(old, new)
                # 3. 去掉开头的 斜杠
                image_slash = replace_image_path.strip('\\/')
                json_slash = replace_json_path.strip('\\/')
                archive_image_path = os.path.join(out_dir, image_slash)
                archive_json_path = os.path.join(out_dir, json_slash)
            labelme_dir = os.path.split(data_info['labelme_dir'])[-1]
            image_dir = os.path.split(data_info['image_dir'])[-1]
            # 针对李世娇，把00.labelme修改成01.labelme后无法查看问题，临时添加
            if '00.labelme' in data_info['labelme_dir']:
                archive_json_path = archive_json_path.replace(labelme_dir, '01.labelme')
            os.makedirs(archive_json_path, exist_ok=True)
            os.makedirs(archive_image_path, exist_ok=True)
            if data_info['image_file'] is not None:
                image_path = os.path.join(data_info['image_dir'], data_info['image_file'])
                try:
                    shutil.copy(image_path, archive_image_path)
                except Exception as e:
                    continue
                if data_info['labelme_file'] is not None:
                    save_json_path = os.path.join(archive_json_path, data_info['labelme_file'])
                    with open(save_json_path, "w", encoding='UTF-8') as labelme_fp:  # 以写入模式打开这个文件
                        json.dump(data_info['labelme_info'], labelme_fp, indent=4, cls=Encoder)  # 从新写入这个文件，把之前的覆盖掉（保存）

    def visualization(self, output_dir, replaces={}):
        """
        可视化保存图片方法实现
        :param output_dir:
        :param replaces:
        """
        global left_top, right_bottom, archive_image_path
        # os.makedirs(output_dir, exist_ok=True)
        for data_info in self.data_infos:
            image = os.path.join(data_info['image_dir'], data_info['image_file'])
            # output_image = os.path.join(output_dir, data_info['image_file'])
            img = cv2.imread(image)
            for old, new in replaces.items():
                replace_image_path = data_info['image_dir'].replace(old, new)
                # replace_json_path = data_info['labelme_dir'].replace(old, new)
                # 3. 去掉开头的 斜杠
                image_slash = replace_image_path.strip('\\/')
                # json_slash = replace_json_path.strip('\\/')
                archive_image_path = os.path.join(output_dir, image_slash)
                os.makedirs(archive_image_path, exist_ok=True)
                # archive_json_path = os.path.join(output_dir, json_slash)
            output_image = os.path.join(archive_image_path, data_info['image_file'])

            if data_info['labelme_info'] is not None:
                for shape in data_info['labelme_info']['shapes']:
                    # 左上角坐标点
                    left_top = (list(map(int, shape['points'][0])))
                    # 右下角坐标点
                    right_bottom = (list(map(int, shape['points'][1])))
                    # (0, 255, 255) => rgb 颜色,3 => 粗细程度,img => 图片数据,
                    cv2.rectangle(img, left_top, right_bottom, (0, 255, 255), 1)
                    # 保存图片
                    cv2.imwrite(output_image, img)

    def rename(self, rename):
        """
        重命名标签类别
        :param rename:
        """
        for data_info in self.data_infos:
            if data_info['labelme_info'] is not None:
                for shape in data_info['labelme_info']['shapes']:
                    # 修改标签类别名称
                    if shape['label'] == 'perosn':
                        print(shape['label'])
                    if shape['label'] in rename.get('name_classes'):
                        shape['label'] = rename.get('name_classes').get(shape['label'])

    def __add__(self, other):
        """
        类别筛选合并操作方法
        :param other:
        :return:
        """
        for index, info in enumerate(other.data_infos):
            # 找出info中的image_file，并把self.data_infos集合传入find(),用于查询other.data_infos中的元素是否在self.data_infos，不在就把other.data_infos元素追加到self.data_infos
            self.find(info['image_file'], self.data_infos)
            # 打印相等与不相等的索引下标
            # print(self.find(info['image_file'], self.data_infos))
            if self.find(info['image_file'], self.data_infos) is None:
                # 如果两个对象中的元素相等就labelme相加，不等的元素直接追加后保存
                self.data_infos.append(info)
                # 挑选不同的labelme元素进行追加
                self.different.append(info)
            if self.data_infos[self.find(info['image_file'], self.data_infos)].get('labelme_info') is None:
                continue
            # 针对需要追加的数据，如果没有shapes就不追加，直接跳过
            if info.get('labelme_info') is None:
                continue
            else:
                # print(self.data_infos[self.find(info['image_file'], self.data_infos)].get('labelme_info').get('shapes'))
                # print(info.get('labelme_info').get('shapes'))
                # print()
                self.data_infos[self.find(info['image_file'], self.data_infos)].get('labelme_info').get(
                    'shapes').extend(info.get('labelme_info').get('shapes'))
        return self

    # @update_property
    def __call__(self, filter_empty, only_empty=False, *args, **kwargs):
        """
        类别筛选功能实现。一个类实例也可以变成一个可调用对象，只需要实现一个特殊方法__call__()
        :param filter_empty: 默认False，保留背景类。设置为True，不保留背景类。
        :param only_empty: 默认False，不保留背景类。设置为True，只筛选背景类。
        :param args: 传入参数可为，数字、字符串
        :param kwargs: 传入参数可为，列表、元组、集合、字典。当前传入列表，参数有name_classes（类别）、shapes_type（形状）
        :return:
        """
        # print(kwargs.get('name_classes'))
        # print(kwargs.get('filter_flags'))
        print(only_empty)
        global name_classes, shapes_type
        self.self_bak.filter_empty = filter_empty
        self.self_bak.only_empty = only_empty
        # 如果filter_empty和only_empty都为真，则报错
        if filter_empty and only_empty:
            assert not filter_empty, '即不需要背景类，同时又只保留背景类条件冲突，请核对业务需求重新传入参数{}'.format(filter_empty)
        # 只留背景类，把不是背景类的删除掉。
        if only_empty:
            def data_info_only_empty(data_info):
                # 如果不是背景类就剔除
                if not data_info['background']:
                    return
                return data_info

            if self.self_bak.only_empty:
                # print(self.self_bak.data_infos)
                self.self_bak.data_infos = list(filter(data_info_only_empty, self.self_bak.data_infos))
                self.self_bak.__update_property__()
                return self.self_bak
        else:
            # 先判断筛选条件是否合法，不合法终止运行。优先判断传入的类型是否为list
            if kwargs.get('name_classes'):
                name_classes = kwargs.get('name_classes')
                # 不是列表赋值成列表，传入的类别是否存在，
                if isinstance(name_classes, str):
                    name_classes = [name_classes]
                if not set(name_classes).issubset(set(self.name_classes)):
                    warnings.warn(
                        '传入的name_classes={} 不是self.name_classes={}的子集'.format(name_classes, self.name_classes))
            if kwargs.get('shapes_type'):
                shapes_type = kwargs.get('shapes_type')
                if isinstance(shapes_type, str):
                    shapes_type = [shapes_type]
                if not set(shapes_type).issubset(set(self.self_bak.shape_type)):
                    warnings.warn(
                        '传入的type_shapes={} 不是self.shape_type={}的子集'.format(shapes_type, self.self_bak.shape_type))
            if kwargs.get('filter_flags'):
                filter_flags = kwargs.get('filter_flags')
                if isinstance(filter_flags, str):
                    filter_flags = dict(filter_flags)
                if not set(filter_flags).issubset(set(self.flags)):
                    warnings.warn('filter_flags={} 不是self.flags={}的子集'.format(filter_flags, self.flags))

            def data_info_filter(data_info):
                if data_info['background']:
                    return
                return data_info

            if self.self_bak.filter_empty:
                self.self_bak.data_infos = list(filter(data_info_filter, self.self_bak.data_infos))
            if kwargs.get('name_classes'):
                # 筛选过程中只保留当前操作类别，其余不需要的类别不留
                self.self_bak._del_class(self.self_bak.data_infos, name_classes, filter_flags)
                self.self_bak.__update_property__()
                return self.self_bak

        # if kwargs.get('shapes_type'):
        # 该方法没有实现
        # self._del_type(self.data_infos, shapes_type)

    def self2labelme(self):
        # 子类没有实现就报错
        raise NotImplementedError



import os
import cv2


class Split(object):
    @classmethod
    def get_videos_path(cls, root_dir, file_formats):
        # dir_name_list = list()  # 文件夹名称
        # files_name = list()  # 文件名称
        # path_name = list()  # 文件夹路径
        file_path_name = list()  # 文件路径
        for root, dirs, files in os.walk(root_dir, topdown=True):
            dirs.sort()
            files.sort()
            # for dir_name in dirs:
            # if dir_name != '00.images' and dir_name != '01.labelme':
            # dir_name_list.append(dir_name)
            # 遍历文件名称列表
            for file in files:
                # 获取文件后缀
                file_suffix = os.path.splitext(file)[-1]
                # 如果读取的文件后缀，在指定的后缀列表中，则返回真继续往下执行
                if file_suffix in file_formats:
                    # 如果文件在文件列表中，则返回真继续往下执行
                    # files_name.append(file)
                    # path_name.append(root)
                    file_path_name.append(os.path.join(root, file))
        return file_path_name

    # def __call__(self, q, video_path, interval, output_dir):
    #     os.makedirs(output_dir, exist_ok=True)
    #     video = cv2.VideoCapture(video_path)
    #     cur_frame = 0
    #     num = 1
    #     while True:
    #         ret, frame = video.read()
    #         if not ret:
    #             break
    #         if cur_frame % int(interval) == 0:
    #             image_name = '{:0>8d}.jpg'.format(cur_frame)
    #             img_dir = os.path.join(output_dir, image_name)
    #             cv2.imencode('.jpg', frame)[1].tofile(img_dir)
    #             num += 1
    #         cur_frame += 1
    #     # 如果提取视频帧完成，那么就向队列中写入一个消息，表示已经完成
    #     q.put(video_path)

    @classmethod
    def video_loader(cls, q, video_path, interval, output_dir, filename_format):
        os.makedirs(output_dir, exist_ok=True)
        video = cv2.VideoCapture(video_path)
        cur_frame = 0
        num = 1
        while True:
            ret, frame = video.read()
            if not ret:
                break
            if cur_frame % int(interval) == 0:
                image_name = filename_format.format(cur_frame)
                img_dir = os.path.join(output_dir, image_name)
                cv2.imencode('.jpg', frame)[1].tofile(img_dir)
                num += 1
            cur_frame += 1
        # 如果提取视频帧完成，那么就向队列中写入一个消息，表示已经完成
        q.put(video_path)
